@@ -25,6 +25,10 @@ The complete XML schema for KML is located at
 http://schemas.opengis.net/kml/.
 
 """
+try:
+    import urlparse
+except ImportError:
+    import urllib.parse as urlparse
 
 from .geometry import Point, LineString, Polygon
 from .geometry import MultiPoint, MultiLineString, MultiPolygon
@@ -42,21 +46,15 @@ import dateutil.parser
 import logging
 logger = logging.getLogger('fastkml.kml')
 
-
 from .config import etree
 
 from .base import _BaseObject
 
 from .styles import StyleUrl, Style, StyleMap, _StyleSelector
 
-try:
-    import fastkml.atom as atom
-    import fastkml.gx as gx
-    import fastkml.config as config
-except  ImportError:
-    import atom
-    import gx
-    import config
+import fastkml.atom as atom
+import fastkml.gx as gx
+import fastkml.config as config
 
 try:
     unicode
@@ -72,8 +70,14 @@ class KML(object):
     ns = None
 
     def __init__(self, ns=None):
+        """ The namespace (ns) may be empty ('') if the 'kml:' prefix is
+        undesired. Note that all child elements like Document or Placemark need
+        to be initialized with empty namespace as well in this case.
+
+        """
         self._features =[]
-        if ns == None:
+
+        if ns is None:
             self.ns = config.NS
         else:
             self.ns = ns
@@ -103,6 +107,13 @@ class KML(object):
 
     def etree_element(self):
         root = etree.Element('%skml' % self.ns)
+
+        # self.ns may be empty, which leads to unprefixed kml elements.
+        # However, in this case the xlmns should still be mentioned on the kml
+        # element, just without prefix.
+        if not self.ns:
+            root.set('xmlns', config.NS[1:-1])
+
         for feature in self.features():
             root.append(feature.etree_element())
         return root
@@ -233,10 +244,13 @@ class _Feature(_BaseObject):
     # A given KML Feature can contain a combination of these types of
     # custom data.
     #
+    # (2) is already implemented, see UntypedExtendedData
+    #
     # <Metadata> (deprecated in KML 2.2; use <ExtendedData> instead)
+    extended_data = None
 
     def __init__(self, ns=None, id=None, name=None, description=None,
-                styles=None, styleUrl=None):
+                styles=None, styleUrl=None, extended_data=None):
         super(_Feature, self).__init__(ns, id)
         self.name=name
         self.description=description
@@ -245,6 +259,7 @@ class _Feature(_BaseObject):
         if styles:
             for style in styles:
                 self.append_style(style)
+        self.extended_data = extended_data
 
     @property
     def styleUrl(self):
@@ -368,7 +383,6 @@ class _Feature(_BaseObject):
             else:
                 raise TypeError
 
-
     def etree_element(self):
         element = super(_Feature, self).etree_element()
         if self.name:
@@ -391,10 +405,10 @@ class _Feature(_BaseObject):
             if isinstance(self._snippet, basestring):
                 snippet.text = self._snippet
             else:
-                assert (isinstance(self._snippet[0], basestring))
-                snippet.text = self._snippet[0]
-                if self._snippet[1]:
-                    snippet.set('maxLines', str(self._snippet[1]))
+                assert (isinstance(self._snippet['text'], basestring))
+                snippet.text = self._snippet['text']
+                if self._snippet.get('maxLines'):
+                    snippet.set('maxLines', str(self._snippet['maxLines']))
         if (self._time_span is not None) and (self._time_stamp is not None):
             raise ValueError('Either Timestamp or Timespan can be defined, not both')
         elif self._time_span is not None:
@@ -405,68 +419,75 @@ class _Feature(_BaseObject):
             element.append(self._atom_link.etree_element())
         if self._atom_author is not None:
             element.append(self._atom_author.etree_element())
+        if self.extended_data is not None:
+            element.append(self.extended_data.etree_element())
         return element
 
 
     def from_element(self, element):
-        if self.ns + self.__name__ != element.tag:
-            raise TypeError
-        else:
-            if element.get('id'):
-                self.id = element.get('id')
-            name = element.find('%sname' %self.ns)
-            if name is not None:
-                self.name = name.text
-            description = element.find('%sdescription' %self.ns)
-            if description is not None:
-                self.description = description.text
-            visibility = element.find('%svisibility' %self.ns)
-            if visibility is not None:
-                self.visibility = int(visibility.text)
-            isopen = element.find('%sopen' %self.ns)
-            if isopen is not None:
-                self.isopen = int(isopen.text)
-            styles = element.findall('%sStyle' % self.ns)
-            for style in styles:
-                s = Style(self.ns)
-                s.from_element(style)
-                self.append_style(s)
-            styles = element.findall('%sStyleMap' % self.ns)
-            for style in styles:
-                s = StyleMap(self.ns)
-                s.from_element(style)
-                self.append_style(s)
-            style_url = element.find('%sstyleUrl' % self.ns)
-            if style_url is not None:
-                s = StyleUrl(self.ns)
-                s.from_element(style_url)
-                self._styleUrl = s
-            snippet = element.find('%sSnippet' % self.ns)
-            if snippet is not None:
-                self._snippet = ('',0)
-                self._snippet[0] = snippet.text
-                if snippet.get('maxLines'):
-                    self._snippet[1] = int(snippet.get('maxLines'))
-            timespan = element.find('%sTimeSpan' % self.ns)
-            if timespan is not None:
-                s = TimeSpan()
-                s.from_element(timespan)
-                self._time_span = s
-            timestamp = element.find('%sTimeStamp' % self.ns)
-            if timestamp is not None:
-                s = TimeStamp()
-                s.from_element(timestamp)
-                self._time_stamp = s
-            atom_link = element.find('%slink' % atom.NS)
-            if atom_link is not None:
-                s = atom.Link()
-                s.from_element(atom_link)
-                self._atom_link = s
-            atom_author = element.find('%sauthor' % atom.NS)
-            if atom_author is not None:
-                s = atom.Author()
-                s.from_element(atom_author)
-                self._atom_author = s
+        super(_Feature, self).from_element(element)
+        name = element.find('%sname' %self.ns)
+        if name is not None:
+            self.name = name.text
+        description = element.find('%sdescription' %self.ns)
+        if description is not None:
+            self.description = description.text
+        visibility = element.find('%svisibility' %self.ns)
+        if visibility is not None:
+            self.visibility = int(visibility.text)
+        isopen = element.find('%sopen' %self.ns)
+        if isopen is not None:
+            self.isopen = int(isopen.text)
+        styles = element.findall('%sStyle' % self.ns)
+        for style in styles:
+            s = Style(self.ns)
+            s.from_element(style)
+            self.append_style(s)
+        styles = element.findall('%sStyleMap' % self.ns)
+        for style in styles:
+            s = StyleMap(self.ns)
+            s.from_element(style)
+            self.append_style(s)
+        style_url = element.find('%sstyleUrl' % self.ns)
+        if style_url is not None:
+            s = StyleUrl(self.ns)
+            s.from_element(style_url)
+            self._styleUrl = s
+        snippet = element.find('%sSnippet' % self.ns)
+        if snippet is not None:
+            self._snippet = {'text': snippet.text}
+            if snippet.get('maxLines'):
+                self._snippet['maxLines'] = int(snippet.get('maxLines'))
+        timespan = element.find('%sTimeSpan' % self.ns)
+        if timespan is not None:
+            s = TimeSpan(self.ns)
+            s.from_element(timespan)
+            self._time_span = s
+        timestamp = element.find('%sTimeStamp' % self.ns)
+        if timestamp is not None:
+            s = TimeStamp(self.ns)
+            s.from_element(timestamp)
+            self._time_stamp = s
+        atom_link = element.find('%slink' % atom.NS)
+        if atom_link is not None:
+            s = atom.Link()
+            s.from_element(atom_link)
+            self._atom_link = s
+        atom_author = element.find('%sauthor' % atom.NS)
+        if atom_author is not None:
+            s = atom.Author()
+            s.from_element(atom_author)
+            self._atom_author = s
+        extended_data = element.find('%sExtendedData' % self.ns)
+        if extended_data is not None:
+            if extended_data.find('%sData' % self.ns):
+                x = UntypedExtendedData(self.ns)
+                x.from_element(extended_data)
+                self.extended_data = x
+            else:
+                logger.warn(
+                    'arbitrary or typed extended data is not yet supported'
+                )
 
 
 
@@ -483,8 +504,9 @@ class _Container(_Feature):
 
     _features = []
 
-    def __init__(self, ns=None, id=None, name=None, description=None):
-        super(_Container, self).__init__(ns, id, name, description)
+    def __init__(self, ns=None, id=None, name=None, description=None,
+                styles=None, styleUrl=None):
+        super(_Container, self).__init__(ns, id, name, description, styles, styleUrl)
         self._features =[]
 
     def features(self):
@@ -534,6 +556,12 @@ class Document(_Container):
             feature.from_element(placemark)
             self.append(feature)
 
+    def get_style_by_url(self, styleUrl):
+        id = urlparse.urlparse(styleUrl).fragment
+        for style in self.styles():
+            if style.id == id:
+                return style
+
 
 class Folder(_Container):
     """
@@ -765,3 +793,72 @@ class TimeSpan(_TimePrimitive):
             raise ValueError("Either begin, end or both must be set")
         #TODO test if end > begin
         return element
+
+
+class UntypedExtendedData(_BaseObject):
+    """ Represents a list of untyped name/value pairs. See docs:
+
+    -> 'Adding Untyped Name/Value Pairs'
+       https://developers.google.com/kml/documentation/extendeddata
+
+    """
+    __name__ = 'ExtendedData'
+
+    def __init__(self, ns=None, id=None, elements=None):
+        super(UntypedExtendedData, self).__init__(ns, id)
+        self.elements = elements or []
+
+    def etree_element(self):
+        element = super(UntypedExtendedData, self).etree_element()
+
+        for subelement in self.elements:
+            element.append(subelement.etree_element())
+
+        return element
+
+    def from_element(self, element):
+        super(UntypedExtendedData, self).from_element(element)
+        self.elements = []
+
+        for subelement in element:
+            el = UntypedExtendedDataElement(self.ns)
+            el.from_element(subelement)
+
+            self.elements.append(el)
+
+
+class UntypedExtendedDataElement(_BaseObject):
+    """ Represents an untyped name/value pair with optional display name. """
+
+    __name__ = 'Data'
+
+    def __init__(self, ns=None, id=None, name=None, value=None, display_name=None):
+        super(UntypedExtendedDataElement, self).__init__(ns, id)
+
+        self.name = name
+        self.value = value
+        self.display_name = display_name
+
+    def etree_element(self):
+        element = super(UntypedExtendedDataElement, self).etree_element()
+
+        element.set('name', self.name)
+
+        value = etree.SubElement(element, "%svalue" % self.ns)
+        value.text = self.value
+
+        if self.display_name:
+            display_name = etree.SubElement(element, "%sdisplayName" % self.ns)
+            display_name.text = self.display_name
+
+        return element
+
+    def from_element(self, element):
+        super(UntypedExtendedDataElement, self).from_element(element)
+
+        self.name = element.get('name')
+        self.value = element.find('%svalue' % self.ns).text
+
+        display_name = element.find('%sdisplayName' % self.ns)
+        if display_name is not None:
+            self.display_name = display_name.text
